function PI = delayiqc(type,tau,W)
% function PI = delayiqc(type,tau,W)
%
% Construct an IQC multiplier PI for a constant, SISO time delay. The
% multiplier is valid for Stau(v):=Dtau(v)-v which represents the
% deviation of the delayed signal from tne nominal undelayed signal.
% This file constructs three types of multipliers that correspond to
% the quadratic constraints listed in the paper by Pfifer and Seiler.
%
% Inputs
%  type: IQC multiplier type.  This can be:
%     -type==1: PI is |W|^2*[0 -1; -1 -1].
%         This is a circle IQC with center = -1 and radius = 1.
%     -type==2: PI is |W|^2*[|phi|^2 0; 0 -1]
%         This is a circle IQC with center = 0 and radius = |phi|. 
%     -type==3: PI is |W|^2*[0 phi'; phi -1]
%         This is a circle IQC with center = phi and radius = |phi|. 
%  tau: Time delay value used to construct the multiplier.
%  W: Optional stable, minimum phase transfer function used to weight the
%         IQC multiplier by frequency. [Default is W=1]
%
% Output
%  PI:  Parahermitian IQC multiplier for Stau as a ss object.
%
% 12/23/2013 PJS Initial Coding

% XXX PJS: Add error checking
if nargin==2
    W=1;
end

% Construct multiplier for Stau by type
switch type
    case 1  
        % This is a circle IQC with center = -1 and radius = 1.
        PI = ss([0 -1;-1 -1]);
    case 2
        % This is a circle IQC with center = 0 and radius = |phi|.     
        b = sqrt(50);
        c = sqrt(12.5);
        a = sqrt(6.5+2*b);
        phi = 2*tf([tau^2, c*tau, 1e-6],[tau^2, a*tau, b]);
        PI = ss([phi'*phi, 0; 0 -1]);
    case 3
        % This is a circle IQC with center = phi and radius = |phi|.     
        A21 = tf([-2.1932, 9.0190/tau, 0.0885/tau^2], ...
            [1 -5.6380/tau -17.0199/tau^2]);
        PI = ss([0 A21'; A21 -1]);
end

% Include frequency weighting
PI = (W'*W)*PI;
