function [g,info] = lpviqcgain(G,PI)
% function [g,info] = lpviqcgain(G,PI)
%
% Compute bound on gain of Fu(G,Delta) where G is an LTI system and Delta
% is an operator described by IQCs PI.
%
% Inputs
%  G: Linear time-invariant system.
%  PI:  Parahermitian IQC multipliers as a TF/SS object. PI(:,:,i) is
%       the i^th multiplier for Delta. This function assumes Delta
%       is square m-by-m and hence PI should be m-by-m-Niqc where
%       Niqc is the number of IQCs.
%
% Outputs
%  g: Bound on the gain of Fu(G,Delta)
%  info:  Structure with information regarding solution of dissipation
%         inequality. Structure includes the fields:
%    -P:  Symmetric matrix used to construct the storage function.
%    -lamopt: Coefficients for optimal conic combination of IQC multipliers
%    -PIopt: Optimal multiplier obtained as the conic combination
%          PIopt = sum_i lam(i) PI(:,:,i)
%

% 12/19/2013 PJS Initial Coding

%% Dimensions
szG = size(G);
if numel(szG) == 2
    szG = [szG 1];
end
nmod = szG(3); % # of model in the G array.
nve = szG(1);  % # of outputs of G = nv + ne
nwd = szG(2);  % # of inputs of G = nw + nd

Niqc = size(PI,3);    % # of IQCs
szPI = size(PI,1);
nv = floor(szPI/2);   % # of inputs to Delta
nw = nv;              % # of outputs of Delta (Delta assumed to be square)
nd = nwd-nw;   % # of disturbances
ne = nve-nv;   % # of errors

%% Compute hard factorization for each IQC multiplier
Psi = cell(Niqc,1);
M = cell(Niqc,1);
for i=1:Niqc
    [Psi{i},M{i}] = jspec(PI(:,:,i));
end

%%
% Construct extended system Pext including dynamics of plant G and
% all auxiliary systems Psi. The I/O relationship for Pext is:
%     [z1; ...; zNiqc; e] = Pext*[w; d]
% where zj is the output of the auxiliary system Psi for the j^th IQC.

% Stack all auxiliary systems:
%      [z1; ...; zNiqc] = AllPsi*[v; w]
% zidx is a cell array of the output indices for zi.
AllPsi = vertcat(Psi{:});
nz = size(AllPsi,1);

zptr = 0;
zidx = cell(Niqc,1);
for i=1:Niqc
    zidx{i} = zptr + (1:2*nv); 
    zptr = zptr + 2*nv;
end

% Form mapping [z;e] = AllPsiI*[v;w;e] 
AllPsiI = blkdiag(AllPsi,eye(ne));

% Form mapping [v;w;e]=Gw*[w;d] 
GI = [G; eye(nwd)];
vweidx = [1:nv, (nv+ne)+(1:nw), nv+(1:ne)];
Gw = GI(vweidx,:);

% Form [z;e]=Pext*[w;d] from the product AllPsi*M
% Pext = ss( zeros(nz+ne,nwd,nmod) );
for i=1:nmod
    Gwi = Gw(:,:,i);
    Pext(:,:,i) = AllPsiI*Gwi;
end

% Partition state matrices of Pext based on inputs [w;d] and outputs [z;e]
[A,B,C,D] = ssdata(Pext);
nx = size(A,1);
Ce = C(end-ne+1:end,:,:);
De = D(end-ne+1:end,:,:);

%%
% Initialize LMI and define variables
setlmis([])
P = lmivar(1,[nx 1]);
for i = 1:Niqc
    lam(i) = lmivar(1,[1 1]);
end
[gsq,ndec] = lmivar(1,[1 1]);
gsqI = lmivar(3,blkdiag(zeros(nx+nw),ndec*eye(nd)));

%%
% Specify LMIs

% LMI for Dissipation Inequality
for k = 1:nmod
    Ak = A(:,:,k);
    Bk = B(:,:,k);
    Cek = Ce(:,:,k);
    Dek = De(:,:,k);
    
    lmiterm([k 1 1 P],[eye(nx); zeros(nw+nd,nx)],[Ak Bk],'s');
    lmiterm([-k 1 1 gsqI],1,1);
    lmiterm([k 1 1 0],[Cek Dek]'*[Cek Dek]);
    
    for i=1:Niqc
        % Grab i1^th IQC
        Mi = M{i};
        Cyi = C(zidx{i},:,k);
        Dyi = D(zidx{i},:,k);
        Mtil = [Cyi Dyi]'*Mi*[Cyi Dyi];
        lmiterm([k 1 1 lam(i)],Mtil,1,'s');
    end
end

% LMI to enforce P>=0
lmiterm([-(nmod+1) 1 1 P],1,1);

% LMI to enforce scalings to be non-negative
for i = 1:Niqc
    lmiterm([-(i+nmod+1) 1 1 lam(i)],1,1);
end

%% Solve SDP: min gamsq subject to LMI constraints
lmisys = getlmis;
c = zeros(ndec,1);
c(end) = 1;

opt = [0 0 0 0 1];
[copt,xopt] = mincx(lmisys,c,opt);

%% Store solution
g = [];
info.P = [];
info.lamopt = [];
info.PIopt  = ss([]);
if ~isempty(xopt)
    g = sqrt(copt);
    info.P = dec2mat(lmisys,xopt,P);
    lamopt = zeros(Niqc,1);
    PIopt = ss(zeros(nv+nw,nv+nw));
    for i = 1:Niqc
        lamopt(i) = dec2mat(lmisys,xopt,lam(i));
        PIopt = PIopt + lamopt(i)*PI(:,:,i);
    end
    info.lamopt = lamopt;
    info.PIopt = PIopt;
end
